@@ -8,8 +8,6 @@ import tempfile
 import time
 from typing import Any, Dict, List, Mapping, Tuple
 
-from boto3 import Session
-
 from localstack.constants import (
     AWS_REGION_US_EAST_1,
     DEFAULT_BUCKET_MARKER_LOCAL,
@@ -17,6 +15,7 @@ from localstack.constants import (
     DEFAULT_LAMBDA_CONTAINER_REGISTRY,
     DEFAULT_PORT_EDGE,
     DEFAULT_SERVICE_PORTS,
+    ENV_INTERNAL_TEST_RUN,
     FALSE_STRINGS,
     INSTALL_DIR_INFRA,
     LOCALHOST,
@@ -189,15 +188,6 @@ def load_environment(profile: str = None):
     dotenv.load_dotenv(path, override=False)
 
 
-def has_docker():
-    try:
-        with open(os.devnull, "w") as devnull:
-            subprocess.check_output("docker ps", stderr=devnull, shell=True)
-        return True
-    except Exception:
-        return False
-
-
 def is_linux():
     return platform.system() == "Linux"
 
@@ -422,14 +412,6 @@ else:
 # additional CLI commands, can be set by plugins
 CLI_COMMANDS = {}
 
-# set of valid regions
-VALID_PARTITIONS = set(Session().get_available_partitions())
-VALID_REGIONS = set()
-for partition in VALID_PARTITIONS:
-    for region in Session().get_available_regions("sns", partition):
-        VALID_REGIONS.add(region)
-
-
 # determine IP of Docker bridge
 if not DOCKER_BRIDGE_IP:
     DOCKER_BRIDGE_IP = "172.17.0.1"
@@ -534,6 +516,9 @@ LAMBDA_DOCKER_DNS = os.environ.get("LAMBDA_DOCKER_DNS", "").strip()
 # additional flags passed to Lambda Docker run/create commands
 LAMBDA_DOCKER_FLAGS = os.environ.get("LAMBDA_DOCKER_FLAGS", "").strip()
 
+# prebuild images before execution? Increased cold start time on the tradeoff of increased time until lambda is ACTIVE
+LAMBDA_PREBUILD_IMAGES = is_env_true("LAMBDA_PREBUILD_IMAGES")
+
 # default container registry for lambda execution images
 LAMBDA_CONTAINER_REGISTRY = (
     os.environ.get("LAMBDA_CONTAINER_REGISTRY", "").strip() or DEFAULT_LAMBDA_CONTAINER_REGISTRY
@@ -559,12 +544,8 @@ S3_SKIP_SIGNATURE_VALIDATION = is_env_not_false("S3_SKIP_SIGNATURE_VALIDATION")
 TEST_IAM_USER_ID = str(os.environ.get("TEST_IAM_USER_ID") or "").strip()
 TEST_IAM_USER_NAME = str(os.environ.get("TEST_IAM_USER_NAME") or "").strip()
 
-# whether to use Lambda functions in a Docker container
+# user-defined lambda executor mode
 LAMBDA_EXECUTOR = os.environ.get("LAMBDA_EXECUTOR", "").strip()
-if not LAMBDA_EXECUTOR:
-    LAMBDA_EXECUTOR = "docker"
-    if not has_docker():
-        LAMBDA_EXECUTOR = "local"
 
 # Fallback URL to use when a non-existing Lambda is invoked. If this matches
 # `dynamodb://<table_name>`, then the invocation is recorded in the corresponding
@@ -590,6 +571,9 @@ LAMBDA_TRUNCATE_STDOUT = int(os.getenv("LAMBDA_TRUNCATE_STDOUT") or 2000)
 # For example: "my-first-stream:1,my-other-stream:2,my-last-stream:1"
 KINESIS_INITIALIZE_STREAMS = os.environ.get("KINESIS_INITIALIZE_STREAMS", "").strip()
 
+# KMS provider - can be either "local-kms" or "moto"
+KMS_PROVIDER = (os.environ.get("KMS_PROVIDER") or "").strip() or "moto"
+
 # URL to a custom OpenSearch/Elasticsearch backend cluster. If this is set to a valid URL, then localstack will not
 # create OpenSearch/Elasticsearch cluster instances, but instead forward all domains to the given backend.
 OPENSEARCH_CUSTOM_BACKEND = (
@@ -696,6 +680,11 @@ CONFIG_ENV_VARS = [
 ]
 
 
+def is_local_test_mode() -> bool:
+    """Returns True if we are running in the context of our local integration tests."""
+    return is_env_true(ENV_INTERNAL_TEST_RUN)
+
+
 def collect_config_items() -> List[Tuple[str, Any]]:
     """Returns a list of key-value tuples of LocalStack configuration values."""
     none = object()  # sentinel object
