@@ -10,15 +10,16 @@ from typing import Dict, List, Union
 
 import boto3
 from localstack_client.config import get_service_port
-from moto import core as moto_core
+from moto.core import BaseModel
+from moto.core.models import InstanceTrackerMeta
 
 from localstack import config, constants
 from localstack.constants import ENV_DEV, LOCALSTACK_INFRA_PROCESS, LOCALSTACK_VENV_FOLDER
-from localstack.runtime import hooks
+from localstack.runtime import events, hooks
 from localstack.services import generic_proxy, install, motoserver
 from localstack.services.generic_proxy import ProxyListener, start_proxy_server
 from localstack.services.plugins import SERVICE_PLUGINS, ServiceDisabled, wait_for_infra_shutdown
-from localstack.utils import analytics, common, config_listener, persistence
+from localstack.utils import analytics, config_listener, persistence
 from localstack.utils.analytics import event_publisher
 from localstack.utils.aws.request_context import patch_moto_request_handling
 from localstack.utils.bootstrap import (
@@ -28,21 +29,19 @@ from localstack.utils.bootstrap import (
     log_duration,
     setup_logging,
 )
-from localstack.utils.common import (
+from localstack.utils.files import cleanup_tmp_files
+from localstack.utils.net import get_free_tcp_port, is_port_open
+from localstack.utils.patch import patch
+from localstack.utils.platform import in_docker
+from localstack.utils.run import ShellCommandThread, run
+from localstack.utils.server import multiserver
+from localstack.utils.sync import poll_condition
+from localstack.utils.threads import (
     TMP_THREADS,
-    ShellCommandThread,
-    edge_ports_info,
-    get_free_tcp_port,
-    in_docker,
-    is_linux,
-    is_port_open,
-    poll_condition,
-    run,
+    FuncThread,
+    cleanup_threads_and_processes,
     start_thread,
 )
-from localstack.utils.run import FuncThread
-from localstack.utils.server import multiserver
-from localstack.utils.testutil import is_local_test_mode
 
 # flag to indicate whether signal handlers have been set up already
 SIGNAL_HANDLERS_SETUP = False
@@ -59,8 +58,9 @@ PROXY_LISTENERS = {}
 # set up logger
 LOG = logging.getLogger(__name__)
 
-# event flag indicating the the infrastructure has been started and that the ready marker has been printed
-INFRA_READY = threading.Event()
+# event flag indicating the infrastructure has been started and that the ready marker has been printed
+# TODO: deprecated, use events.infra_ready
+INFRA_READY = events.infra_ready
 
 # event flag indicating that the infrastructure has been shut down
 SHUTDOWN_INFRA = threading.Event()
@@ -99,24 +99,26 @@ def patch_urllib3_connection_pool(**constructor_kwargs):
 
 
 def patch_instance_tracker_meta():
-    """
-    Avoid instance collection for moto dashboard
-    """
+    """Avoid instance collection for moto dashboard"""
 
-    def new_intance(meta, name, bases, dct):
-        cls = super(moto_core.models.InstanceTrackerMeta, meta).__new__(meta, name, bases, dct)
+    if hasattr(InstanceTrackerMeta, "_ls_patch_applied"):
+        return  # ensure we're not applying the patch multiple times
+
+    @patch(InstanceTrackerMeta.__new__, pass_target=False)
+    def new_instance(meta, name, bases, dct):
+        cls = super(InstanceTrackerMeta, meta).__new__(meta, name, bases, dct)
         if name == "BaseModel":
             return cls
         cls.instances = []
         return cls
 
-    moto_core.models.InstanceTrackerMeta.__new__ = new_intance
-
+    @patch(BaseModel.__new__, pass_target=False)
     def new_basemodel(cls, *args, **kwargs):
-        instance = super(moto_core.models.BaseModel, cls).__new__(cls)
+        # skip cls.instances.append(..) which is done by the original/upstream constructor
+        instance = super(BaseModel, cls).__new__(cls)
         return instance
 
-    moto_core.models.BaseModel.__new__ = new_basemodel
+    InstanceTrackerMeta._ls_patch_applied = True
 
 
 def get_multiserver_or_free_service_port():
@@ -266,11 +268,11 @@ def add_service_proxy_listener(api: str, listener: ProxyListener, port=None):
     PROXY_LISTENERS[api] = (api, port or get_service_port(api), listener)
 
 
-def start_local_api(name, port, api, method, asynchronous=False):
+def start_local_api(name, port, api, method, asynchronous=False, listener=None):
     log_startup_message(name)
     if config.FORWARD_EDGE_INMEM:
         port = get_free_tcp_port()
-        PROXY_LISTENERS[api] = (api, port, None)
+        PROXY_LISTENERS[api] = (api, port, listener)
     if asynchronous:
         thread = start_thread(method, port, quiet=True)
         return thread
@@ -279,21 +281,20 @@ def start_local_api(name, port, api, method, asynchronous=False):
 
 
 def stop_infra():
-    if common.INFRA_STOPPED:
+    if events.infra_stopping.is_set():
         return
-    common.INFRA_STOPPED = True
+    # also used to signal shutdown for edge proxy so that any further requests will be rejected
+    events.infra_stopping.set()
 
     event_publisher.fire_event(event_publisher.EVENT_STOP_INFRA)
     analytics.log.event("infra_stop")
 
     try:
-        generic_proxy.QUIET = True
+        generic_proxy.QUIET = True  # TODO: this doesn't seem to be doing anything
         LOG.debug("[shutdown] Cleaning up services ...")
         SERVICE_PLUGINS.stop_all_services()
-        LOG.debug("[shutdown] Cleaning up files ...")
-        common.cleanup(files=True, quiet=True)
         LOG.debug("[shutdown] Cleaning up resources ...")
-        common.cleanup_resources()
+        cleanup_resources()
 
         if config.FORCE_SHUTDOWN:
             LOG.debug("[shutdown] Force shutdown, not waiting for infrastructure to shut down")
@@ -303,11 +304,16 @@ def stop_infra():
         wait_for_infra_shutdown()
         LOG.debug("[shutdown] Infrastructure is shut down")
     finally:
-        SHUTDOWN_INFRA.set()
+        events.infra_stopped.set()
+
+
+def cleanup_resources():
+    cleanup_tmp_files()
+    cleanup_threads_and_processes()
 
 
 def log_startup_message(service):
-    LOG.info("Starting mock %s service on %s ...", service, edge_ports_info())
+    LOG.info("Starting mock %s service on %s ...", service, config.edge_ports_info())
 
 
 def check_aws_credentials():
@@ -331,7 +337,7 @@ def terminate_all_processes_in_docker():
         return
     print("INFO: Received command to restart all processes ...")
     cmd = (
-        'ps aux | grep -v supervisor | grep -v docker-entrypoint.sh | grep -v "make infra" | '
+        'ps aux | grep -v supervisor | grep -v docker-entrypoint.sh | grep -v "bin/localstack" | '
         "grep -v localstack_infra.log | awk '{print $1}' | grep -v PID"
     )
     pids = run(cmd).strip()
@@ -374,26 +380,12 @@ def print_runtime_information(in_docker=False):
 
 
 def start_infra(asynchronous=False, apis=None):
+    events.infra_starting.set()
+
     try:
         os.environ[LOCALSTACK_INFRA_PROCESS] = "1"
 
         is_in_docker = in_docker()
-        # print a warning if we're not running in Docker but using Docker based LAMBDA_EXECUTOR
-        if not is_in_docker and "docker" in config.LAMBDA_EXECUTOR and not is_linux():
-            print(
-                (
-                    "!WARNING! - Running outside of Docker with $LAMBDA_EXECUTOR=%s can lead to "
-                    "problems on your OS. The environment variable $LOCALSTACK_HOSTNAME may not "
-                    "be properly set in your Lambdas."
-                )
-                % config.LAMBDA_EXECUTOR
-            )
-
-        if is_in_docker and not config.LAMBDA_REMOTE_DOCKER and not config.dirs.functions:
-            print(
-                "!WARNING! - Looks like you have configured $LAMBDA_REMOTE_DOCKER=0 - "
-                "please make sure to configure $HOST_TMP_FOLDER to point to your host's $TMPDIR"
-            )
 
         print_runtime_information(is_in_docker)
 
@@ -428,7 +420,6 @@ def start_infra(asynchronous=False, apis=None):
 
 
 def do_start_infra(asynchronous, apis, is_in_docker):
-
     event_publisher.fire_event(
         event_publisher.EVENT_START_INFRA,
         {"d": is_in_docker and 1 or 0, "c": in_ci() and 1 or 0},
@@ -454,7 +445,7 @@ def do_start_infra(asynchronous, apis, is_in_docker):
         os.environ["AWS_REGION"] = config.DEFAULT_REGION
         os.environ["ENV"] = ENV_DEV
         # register signal handlers
-        if not is_local_test_mode():
+        if not config.is_local_test_mode():
             register_signal_handlers()
         # make sure AWS credentials are configured, otherwise boto3 bails on us
         check_aws_credentials()
@@ -498,10 +489,6 @@ def do_start_infra(asynchronous, apis, is_in_docker):
     @log_duration()
     def start_runtime_components():
         from localstack.services.edge import start_edge
-        from localstack.services.internal import LocalstackResourceHandler, get_internal_apis
-
-        # serve internal APIs through the generic proxy
-        ProxyListener.DEFAULT_LISTENERS.append(LocalstackResourceHandler(get_internal_apis()))
 
         # TODO: we want a composable LocalStack runtime (edge proxy, service manager, dns, ...)
         t = start_thread(start_edge, quiet=False)
@@ -530,7 +517,7 @@ def do_start_infra(asynchronous, apis, is_in_docker):
     print(READY_MARKER_OUTPUT)
     sys.stdout.flush()
 
-    INFRA_READY.set()
+    events.infra_ready.set()
     analytics.log.event("infra_ready")
 
     hooks.on_infra_ready.run()
